@@ -121,14 +121,14 @@ listEnsembl <- function(mart = NULL, version = NULL,
   
   host <- NULL
   
-  if(!is.null(mirror) & (!is.null(version) | !is.null(GRCh))){
+  if(!is.null(mirror) && (!is.null(version) || !is.null(GRCh))){
     warning("version or GRCh arguments cannot be used together with the mirror argument.\n", 
             "We will ignore the mirror argument and connect to the main Ensembl site.",
             call. = FALSE) 
     mirror <- NULL
   }
   
-  if(!is.null(version) & !is.null(GRCh)) {
+  if(!is.null(version) && !is.null(GRCh)) {
     stop("version or GRCh arguments cannot be used together.\n", 
          "Please specify only the 'version' or 'GRCh' argument.",
          call. = FALSE) 
@@ -166,7 +166,7 @@ listEnsembl <- function(mart = NULL, version = NULL,
   }
   
   if(is.null(host)) {
-    host = "https://www.ensembl.org"
+    host <- "https://www.ensembl.org"
   }
   
   return(host)
@@ -182,10 +182,10 @@ useEnsembl <- function(biomart, dataset, host,
          "the function listEnsembl()")
   }
 
-  if(tolower(biomart) == "ensembl" | tolower(biomart) == "genes") {
+  if(tolower(biomart) == "ensembl" || tolower(biomart) == "genes") {
     biomart = "ENSEMBL_MART_ENSEMBL"
   }
-  if(tolower(biomart) == "snp" | tolower(biomart) == "snps"){
+  if(tolower(biomart) == "snp" || tolower(biomart) == "snps"){
     biomart = "ENSEMBL_MART_SNP"
   }
   if(tolower(biomart) == "regulation"){
