@@ -6,9 +6,9 @@ Sys.setenv(BIOMART_CACHE = cache)
 context('listMarts()')
 ###############################
 
-test_that("listMarts retruns a data.frame", {
+test_that("listMarts returns a data.frame", {
     
-    ensembl_marts <- listMarts(host = "www.ensembl.org")
+    ensembl_marts <- listMarts(host = "https://www.ensembl.org")
     expect_is(ensembl_marts, class = "data.frame")
     expect_identical(colnames(ensembl_marts),
                      c("biomart", "version"))
@@ -17,7 +17,7 @@ test_that("listMarts retruns a data.frame", {
 
 test_that("Error when archive = TRUE", {
     
-    expect_error(listMarts(host = "www.ensembl.org", archive = TRUE),
+    expect_error(listMarts(host = "https://www.ensembl.org", archive = TRUE),
                  regexp = "Use listEnsemblArchives")
        
 })
